"""
This file contains commands that should be made available to user
code.
"""

import traceback as _traceback
import executive as _executive

# Image things we want to export to the user.
from layout import Anchor as anchor
from image import image as image
from music import music, stop_music
from effects import Alpha as alpha

# This will be filled in with a pointer to the user interface /
# executive object.
ui = None

def prompt(message, length, default=""):
    """
    This prompts the user to input a text string.
    
    @param message: A message that's used to prompt the user.
    @param length: The length of the string that will be input.
    @param default: The initial contents of the prompt box, that
    can be changed by the user.
    """

    return ui.ui_prompt(message, length, default)

def cprompt(message, length, default=""):
    """
    This prompts the user to input a text string. The box is centered
    on the screen.
    
    @param message: A message that's used to prompt the user.
    @param length: The length of the string that will be input.
    @param default: The initial contents of the prompt box, that
    can be changed by the user.
    """

    return ui.ui_center_prompt(message, length, default)

def store(*vars, **kwvars):
    """
    Adds given variable names to the list of variables that will be
    preserved and restored when the game is saved and loaded.

    The parameters should be a list of variable names, as strings.
    """

    ui.store.extend(vars)

    for i in kwvars:
        if i not in ui.store:
            ui.store.extend(i)
        ui.set(i, kwvars[i])

def say(a, b=None):
    """
    This displays dialogue to the screen. If there are two strings,
    the first is considered to be the name of the character speaking,
    and the second is the line of dialog spoken. A single string is
    displayed by itself, without a character name.
    """

    ui.do_dialogue(a, b, False)

def centered(text):
    """
    Shows the text, centered on the string.
    """
    
    ui.ui_centered(text)
    
def quit():
    """
    This terminates the running of the interpreter.
    """
    
    raise _executive.EndGameException()

def menu(label, choices):
    return ui.do_menu(label, choices, False)

def cmenu(label, choices):
    return ui.do_menu(label, choices, True)
    
def save():
    fn = cprompt("Enter the name of a save file.", 32)

    if not fn:
        centered("You have to enter a file name to save to.")
        return

    try:
        ui.do_save(fn + ".save")
        centered("Saved game in '%s'." % fn)
    except:
        _traceback.print_exc()        
        centered("Saving failed.")

def load():

    import os
    saves = [ f[:-5] for f in os.listdir(".") if f[-5:] == ".save" ]

    saves.sort()

    index = cmenu("Choose a save-game to load:\n",
                  [ "(Don't load anything.)\n" ] + saves)

    # TODO: Do something smart if we have more game files than will
    # fit on a single screen.
    
    if index == 0:
        return

    prefix =  saves[index - 1]

    try:
        ui.do_load(prefix + ".save")
    except _executive.FragJumpException:
        raise
    except:
        _traceback.print_exc()
        centered("Loading failed.")
        
