# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
This module contains the code responsible for display management.
"""

import sys
import copy

import pygame
from pygame.locals import *

from params import config
from Numeric import divide

# An event that is used to handle timer ticks.
DISPLAYTIMER = USEREVENT + 1
INPUTTIMER = USEREVENT + 2

# The scene.
screen = None

class Transform(object):

    def __init__(self, t):
        self.x = 0
        self.y = 0
        self.t = t
        self.a = 100

    def move(self, x, y):
        rv = copy.copy(self)
        rv.x += x
        rv.y += y

        return rv

    def alpha(self, a):
        rv = copy.copy(self)
        rv.a = a
        return rv

    def blit(self, src, x, y):

        if self.a != 100:
            src = src.convert_alpha()
            pa = pygame.surfarray.pixels_alpha(src)

            na = pa * self.a
            divide(na, 100, na)
#            na = na / 100
            na = na.astype('b')

            pa[:,:] = na[:,:]

            del na
            del pa
            
        
        screen.blit(src, (x + self.x, y + self.y))

class Showable(object):

    def key_event(self, e):
        return

    def mouse_event(self, e, x, y):
        return

    def construct(self):
        return self

def construct(showable):
    """
    When given a string, constructs an image from that string. When
    given a showable object, calls the construct method of that
    object, which is responsible for calling convert on any showable
    fields.
    """

    if isinstance(showable, str):
        import image
        return image.image(showable)
    else:
        showable.construct()
        return showable

def full_construct(showable):
    """
    First calls construct to build up what is displayed on the
    screen. If the resulting showable isn't the size of the screen,
    anchor it to the bottom left.
    """

    showable = construct(showable)
    
    if showable.size() != (config.screen_width, config.screen_height):
        import layout
        showable = layout.Anchor(showable)

    return showable
    

def init():
    global screen
    pygame.init()
    screen = pygame.display.set_mode((config.screen_width, config.screen_height))
    # pygame.display.toggle_fullscreen()

scene_list = None

def clear():
    global scene_list
    scene_list = [ ]
    redraw(0)           

def show(s):
    scene_list.append(s)
    redraw(0)

def hide(s):
    scene_list.remove(s)
    redraw(0)

def draw():
    trans = Transform(pygame.time.get_ticks())

    for s in scene_list:
        s.draw(trans)

    pygame.display.flip()

redraw_time = sys.maxint

def redraw(t):
    global redraw_time

    if t < redraw_time:
        redraw_time = t
    
def interact(ui):

    global redraw_time

    reverse_scene_list = scene_list[:]
    reverse_scene_list.reverse()
    
    keep_running = True

    pygame.event.set_allowed(DISPLAYTIMER)

    draw()

    while keep_running:

        if not pygame.event.peek([KEYDOWN,MOUSEMOTION,MOUSEBUTTONDOWN]):
            
            scenetime = pygame.time.get_ticks()
            timeleft = redraw_time - scenetime

            if timeleft <= 0:
                redraw_time = sys.maxint 
                draw()

            pygame.time.set_timer(DISPLAYTIMER,
                                  max(redraw_time - scenetime, 10))

        e = pygame.event.wait()

        if e.type == DISPLAYTIMER:
            redraw_time = 0

        if e.type == QUIT:
            # Bail out. TODO make this better somehow.            
            pygame.quit()
            sys.exit(0)

        if e.type == KEYDOWN:

            # Handle escape specially.
            if e.key == K_ESCAPE:
                ui.do_escape()
                continue

            for s in reverse_scene_list:
                kerv = s.key_event(e)

                if kerv is True:
                    keep_running = False
                    break
                elif kerv is False:
                    break

        mouse = None

        if e.type == MOUSEMOTION:
            mouse = (False, e.pos[0], e.pos[1])

        if e.type == MOUSEBUTTONDOWN and e.button == 1:
            mouse = (True, e.pos[0], e.pos[1])

        if mouse:
            for s in reverse_scene_list:
                merv = s.mouse_event(*mouse)

                if merv is True:
                    keep_running = False
                    break
                elif merv is False:
                    break
            
            
