# Copyright 2004 Tom Rothamel <tom-renpy@onegeek.org>
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import cPickle as pickle

from character import Character
import fragment
import commands

class EndGameException:
    pass

class FragJumpException:
    def __init__(self, target):
        self.target = target

class Executive(object):
    """
    @ivar env: The environment that python code executes in.
    """

    def __init__(self):

        commands.ui = self

        self.env = {}
        self.scene = [ ]
        self.store = [ ]
        self.return_stack = [ ]

        self.escape_mode = False

        # Export the commands into the default environment.
        exec "from renpy.commands import *" in self.env


    def eval(self, cmd):
        return eval(cmd, self.env)

    def run(self, frag):

        while frag:
            self.current_frag = frag.name

            try:
                frag = frag.run(self)                
            except FragJumpException, e:
                frag = fragment.lookup(e.target)

            if not frag and self.return_stack:
                frag = fragment.lookup(self.return_stack.pop())

            
    def do_escape(self):
        if self.escape_mode:
            return

        self.escape_mode = True

        try:
            frag = fragment.lookup("escape_menu")        
            while frag:
                frag = frag.run(self)
        finally:
            self.escape_mode = False

    def do_save_return(self, target):
        self.return_stack.append(target.name)

    def do_python(self, cmd):
        try:
            exec cmd in self.env
        except:
            print "Exception occured while executing python code:"
            print cmd
            raise

    def do_dialogue(self, who, what, lookup):

        if not who:
            self.ui_dialogue_1(what % self.env)
            return

        # TODO: Error handling.

        if lookup:
            who = lookup_name(self.env, who)

        if not isinstance(who, str):
            who = who.name

        self.ui_dialogue_2(who, what % self.env)

    def do_menu(self, label, choices, centered):
        if label:
            label = label % self.env

        choices = [ i % self.env for i in choices ]

        if centered:
            return self.ui_center_menu(label, choices)
        else:
            return self.ui_menu(label, choices)
            

    def do_character(self, name):
        self.env[name] = Character(name)
        self.store.append(name)

    def do_scene(self):
        self.scene = [ ]
        self.ui_clear_scene()

    def do_hide(self, thing):

        # return true if a is a prefix of b.
        def prefix(a, b):
            if len(b) < len(a):
                return False

            for av, bv in zip(a, b):
                if av != bv:
                    return False

            return True

        newscene = []

        for t in self.scene:
            if prefix(thing, t):
                self.ui_hide(t)
            else:
                newscene.append(t)
        
        self.scene = newscene

    def do_show(self, thing, with):
        self.do_hide((thing[0],))
        self.scene.append(thing)
        self.ui_show(thing, with)
        
    def do_save(self, fn):

        env = [ (key, lookup_name(self.env, key)) for key in self.store ]
        state = self.current_frag, self.return_stack, self.scene, env

        print repr(env)

        f = file(fn, "w")
        pickle.dump(state, f)
        f.close()

    def do_load(self, fn):
        f = file(fn)
        state = pickle.load(f)
        f.close()

        frag, self.return_stack, self.scene, env = state
        
        for k, v in env:
            store_name(self.env, k, v)

        self.ui_reshow_scene()

        raise FragJumpException(frag)


    def set(self, k, v):
        store_name(self.env, k, v)

def lookup_name(context, name):
    l = name.split('.')
    v = context[l[0]]

    for i in l[1:]:
        v = getattr(v, i)

    return v

def store_name(context, name, value):

    if "." not in name:
        context[name] = value
        return
    
    obj = lookup_name(name[:name.rindex('.')])
    setattr(obj, name[name.rindex('.') + 1:], value)

    
    
